#include <def21535.h>
#define RX_DMA_LENGTH_SOURCE		65535
#define RX_DMA_LENGTH_DESTINATION	1023
#define TX_DMA_LENGTH_SOURCE		1023
#define TX_DMA_LENGTH_DESTINATION	65535

#define RX_MODE			1
#define TX_MODE			2
#define PR_MODE			4
#define	PT_MODE			8

#define	IO_PORT_ADDR	0x28000000		//Bank2 od asych mem, AMS2 effective
/*SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/
/*Program Section*/
.section program;    

/* Init operating mode, stay in supervisor */
SetupOpMode:
	/* int 15 point to user program */
	p0.l	= EVT15 & 0xffff;
	p0.h	= (EVT15 >> 16) & 0xffff;
	r0		= MainProc(Z);
	r0.h	= MainProc;
	[p0]	= r0;
	
	/* Enable int 15 */
	p0.l	= IMASK & 0xffff;
	p0.h	= (IMASK >>16) & 0xffff;
	r0.l	= W[p0];
	r1.l	= EVT_IVG15;
	r0		= r0 | r1;
	W[p0]	= r0.l;

	/* Create int 15 */
	raise 15;

	p0.l	= DeadLoop;
	p0.h	= DeadLoop;
	reti	= p0;
	rti;
	nop;
	nop;

DeadLoop:
	nop;
	jump	DeadLoop;


MainProc:
			
	call	InitRegs;

	/* Initialize Stack */
	SP 		=UserStack (Z);
	SP.h	=UserStack;
	

	/* Enable interrupt,still in superviser mode */
	[--sp]	=reti;

	call	SetupEvtHandler;

	call	SetupCoreTimer;

	/* Setup EBIU */
	call	SetupEBIUAsynchMem;

	/* Init Mode as TX, 0-RX, 1-TX, 2-PR, 3-PT */
	p0.l	= SysMode;
	p0.h	= SysMode;
	r0.l	= RX_MODE;
	W[p0]	= r0.l;

	/* Setup DMA */
	call	InitCommonDMA;
	call	SetupRXDMA;

	/* Setup PF pin, PF0,1,2,3 output */
	r0.l=0x000f;
	p0.l = FIO_DIR & 0xffff;
	p0.h = FIO_DIR >> 16;
	p1.l = FIO_FLAG_S & 0xffff;
	p1.h = FIO_FLAG_S >> 16;
	p2.l = FIO_FLAG_C & 0xffff;
	p2.h = FIO_FLAG_C >> 16;
	W[p0] = r0;

	/* Enable MemDMA sys int . Manual has something wrong, it is clear to enable interrupt*/
	i1.L = SIC_IMASK & 0xffff;
	i1.H = SIC_IMASK >> 16;
	r3	 = [i1];
    bitclr(r3, 19);
	[i1] = r3;
	csync;

	/* Enable core timer interrupt & MemDMA write int */
	i1.L = IMASK & 0xffff;
	i1.H = IMASK >> 16;
   	r3 =  0x2040 (Z);     
    [ i1 ] = r3;
	csync;
	
	/* Start RX DMA operation */
	call	StartRXDMA;

	
	/* Start core timer */
	i0.l = TCNTL & 0xffff;
	i0.h = TCNTL >> 16;
	r4 = 0x0007 (Z);

	r5=0x1;



	/* Testing DMA */
LoopA:
	p1.l	= RXReadDMADp1;
	p1.h	= RXReadDMADp1;

DMA_WAIT1:
	R6 = W[P1];
	cc = bittst(R6,15);
	IF cc JUMP DMA_WAIT1;
	nop;
	bitset(r6,15);
	W[p1] = R6;
	nop;
	p1.l	= RXWriteDMADp1;
	p1.h	= RXWriteDMADp1;
	R6 = W[P1];
	bitset(r6,15);
	W[p1] = R6;
	nop;


	p1.l	= RXReadDMADp2;
	p1.h	= RXReadDMADp2;

DMA_WAIT2:
	R6 = W[P1];
	cc = bittst(R6,15);
	IF cc JUMP DMA_WAIT2;
	nop;
	bitset(r6,15);
	W[p1] = R6;
	nop;
	nop;
	p1.l	= RXWriteDMADp2;
	p1.h	= RXWriteDMADp2;
	R6 = W[P1];
	bitset(r6,15);
	W[p1] = R6;
	nop;
	jump	LoopA;




_EHANDLER:            // Emulation Handler 0
    RTE;

_RHANDLER:            // Reset Handler 1
    RTI;

_NHANDLER:            // NMI Handler 2
    RTN;

_XHANDLER:            // Exception Handler 3

off1:
	nop;
	nop;
	Jump off1;
    RTX;

_HWHANDLER:           // HW Error Handler 5
off2:
	nop;
	nop;
	Jump off2;
    RTI;

_THANDLER:            // Timer Handler 6
    

	[--sp]	= p1;
   	[--sp]	= p2;
	[--sp]	= r5;

	p1.l = FIO_FLAG_S & 0xffff;
	p1.h = FIO_FLAG_S >> 16;

	p2.l = TimerToggle ;
	p2.h = TimerToggle ;

	r5	= W[p2](Z);

	bittgl(r5,0);
	W[p2]	=r5.l;

	CC = r5 == 0x1;
	IF CC JUMP on;
	p1.l = FIO_FLAG_C & 0xffff;
	p1.h = FIO_FLAG_C >> 16;
on: 
	r5	= 0x0001;
	W[p1]=r5;

	r5	= [sp ++];
	p2	= [sp ++];
	p1	= [sp ++];

	RTI;

_RTCHANDLER:		// IVG 7 Handler  (RTC)
	RTI;

_I8HANDLER:			// IVG 8 Handler
    RTI;

_I9HANDLER:			// IVG 9 Handler
    RTI;

_I10HANDLER:		// IVG 10 Handler
    RTI;

_I11HANDLER:		// IVG 11 Handler
	
    RTI;

_I12HANDLER:		// IVG 12 Handler
    RTI;

_I13HANDLER:		// IVG 13 Handler
	[--sp]	= p0;
	[--sp] 	= r0;
	[--sp]	= r1;

	/* Clear MemWDMA interrupt reg */
	p0.l	= MDR_DI & 0xffff;
	p0.h	= (MDR_DI >> 16) & 0xffff;
	r0.l	= W[p0];
	W[p0]	= r0.l;


	/*=======*/
	p0.l	= SysMode;
	p0.h	= SysMode;
	r0.l	= W[p0];
	cc=	bittst(r0, 1);
	if cc Jump TXProcessing;

RXProcessing:
	p0.l		= RXDMACount;
	p0.h		= RXDMACount;
    r0			= [p0];
	r0			+= 1;
	[p0]		= r0;

	p0.l = FIO_FLAG_C & 0xffff;
	p0.h = FIO_FLAG_C >> 16;

	cc = bittst(r0,22);
	if cc jump	RXLedOff;

	p0.l = FIO_FLAG_S & 0xffff;
	p0.h = FIO_FLAG_S >> 16;

RXLedOff:
	r1		= 0x0002;
	W[p0]	= r1;

	cc = bittst(r0,0);
	if cc jump  RXOddFrame;

	p0.l		= RXReadDMADp2;
	p0.h		= RXReadDMADp2;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	p0.l		= RXWriteDMADp2;
	p0.h		= RXWriteDMADp2;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	jump		MemDMAISREnd;

RXOddFrame:
	p0.l		= RXReadDMADp1;
	p0.h		= RXReadDMADp1;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	p0.l		= RXWriteDMADp1;
	p0.h		= RXWriteDMADp1;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

MemDMAISREnd:
	
	r1	= [sp++];
	r0	= [sp++];
	p0	= [sp++];
	rti;

TXProcessing:
	p0.l		= TXDMACount;     
	p0.h		= TXDMACount;
    r0			= [p0];
	r0			+= 1;
	[p0]		= r0;

	p0.l = FIO_FLAG_C & 0xffff;
	p0.h = FIO_FLAG_C >> 16;

	cc = bittst(r0,22);
	if cc jump	TXLedOff;

	p0.l = FIO_FLAG_S & 0xffff;
	p0.h = FIO_FLAG_S >> 16;

TXLedOff:
	r1		= 0x0004;
	W[p0]	= r1;

	cc = bittst(r0,0);
	if cc jump  TXOddFrame;

	p0.l		= TXReadDMADp2;
	p0.h		= TXReadDMADp2;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	p0.l		= TXWriteDMADp2;
	p0.h		= TXWriteDMADp2;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	jump		MemDMAISREnd;

TXOddFrame:
	p0.l		= TXReadDMADp1;
	p0.h		= TXReadDMADp1;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	p0.l		= TXWriteDMADp1;
	p0.h		= TXWriteDMADp1;
	r0			= W[p0];
	bitset(r0,15);
	W[p0]		= r0;

	Jump	MemDMAISREnd;	



_I14HANDLER:
    RTI;

_I15HANDLER:
    RTI;

/////////////////////
/* Subroutine area */
/////////////////////
/*-----------------------------------------/
	Init all register
	Because the reset might not reset 
	the register into a legal value 
/----------------------------------------*/
InitRegs:
	/* Initialize all register file */
	R0 = 0; R1 = 0; R2 = 0; R3 = 0; 
	R4 = 0; R5 = 0; R6 = 0; R7 = 0;		
	P0 = 0; P1 = 0; P2 = 0; P3 = 0; P4 = 0; P5 = 0;		
	I0 = 0 (X); I1 = 0 (X); I2 = 0 (X); I3 = 0 (X);   
	M0 = 0 (X); M1 = 0 (X); M2 = 0 (X); M3 = 0 (X);
	L0 = 0 (X); L1 = 0 (X); L2 = 0 (X); L3 = 0 (X);
	B0 = 0 (X); B1 = 0 (X); B2 = 0 (X); B3 = 0 (X);
	rts;

/*-----------------------------------------/
	Setup all the interrupt handler 
/----------------------------------------*/
SetupEvtHandler:	
	/*===================================*/
	/* Setup Event Vectors and Handlers */
	R0 = 0;
	p0.l = EVT0 & 0xffff;
	p0.h = EVT0 >> 16;
	

	p0	+=4;

	// Reset Handler (Int1)
    R0 = _RHANDLER (Z);
	R0.H = _RHANDLER;  	
    [ P0 ++ ] = R0;

	// NMI Handler (Int2)
    R0 = _NHANDLER (Z);
	R0.H = _NHANDLER;  	
    [ P0 ++ ] = R0;

	// Exception Handler (Int3)
    R0.L = _XHANDLER;
	R0.H = _XHANDLER;  	
    [ P0 ++ ] = R0;
	
	// IVT4 isn't used, Reserved
	[ P0 ++ ] = R0; 	

	// HW Error Handler (Int5)
    R0 = _HWHANDLER (Z);
	R0.H = _HWHANDLER; 	
    [ P0 ++ ] = R0;
	
	// Core Timer Handler (Int6)
    R0 = _THANDLER (Z);
	R0.H = _THANDLER;  
    [ P0 ++ ] = R0;
	
	// IVG7 Handler, (RTC, USB, PCI)
    R0 = _RTCHANDLER (Z);   
	R0.H = _RTCHANDLER; 	
    [ P0 ++ ] = R0;

	// IVG8 Handler (SPORT0,1-RX/TX )
    R0 = _I8HANDLER (Z);  
	R0.H = _I8HANDLER; 	
    [ P0 ++ ] = R0;

	// IVG9 Handler (SPI0,1)
    R0 = _I9HANDLER (Z);  
	R0.H = _I9HANDLER; 	
    [ P0 ++ ] = R0;

	// IVG10 Handler (UART0,1 --RX/TX ) 
    R0 = _I10HANDLER (Z);	
	R0.H = _I10HANDLER;		
    [ P0 ++ ] = R0;

	// IVG11 Handler (Timer 0,1,2)
    R0 = _I11HANDLER (Z);	
	R0.H = _I11HANDLER;		
    [ P0 ++ ] = R0;

	// IVG12 Handler (Programmable flag A/B)
    R0 = _I12HANDLER (Z);	
	R0.H = _I12HANDLER;		
    [ P0 ++ ] = R0;

	// IVG13 Handler (Memory DMA )
    R0 = _I13HANDLER (Z);	
	R0.H = _I13HANDLER;		
    [ P0 ++ ] = R0;

	// IVG14 Handler (Software Interrrupt 0)
    R0 = _I14HANDLER (Z);	
	R0.H = _I14HANDLER;		
    [ P0 ++ ] = R0;

	// IVG15 Handler (Software Interrupt 1)
    R0 = _I15HANDLER (Z);	
	R0.H = _I15HANDLER;		
    [ P0 ++ ] = R0;

	// ???????????
	P0 = EVT_OVERRIDE & 0xffff;
	P0.H = EVT_OVERRIDE >> 16;
    R0 = 0;
    [ P0 ] = R0;

	/*===================================*/
	rts;

/*-----------------------------------------/
	Setup all the interrupt handler 
/----------------------------------------*/
SetupCoreTimer:
	/* Setup core timer */
	i0.l = TCNTL & 0xffff;
	i0.h = TCNTL >> 16;	
	m0=4;
	r0=0x5 (Z);
	[i0++m0] = r0;		//Ctimer Control reg
	csync;

	r1.l=0x0000;
	r1.h=0x0004;
	[i0++m0]=r1;		//Ctimer period
	csync;
	
	r2 = 0x0080 (Z);
	[i0++m0]=r2;		//Ctimer scale
	csync;

	[i0]=r1;			//Ctimer count
	csync;
	
	rts;

	
/*----------------------------------------------*
 *	Name		:	SetupEBIUAsynchMem
 *	Function	:	Setup EBIU 's Aych memory for
 				  	ADC/DAC data
 *	Register 	:
 *		In:			None		
 *		Out:		None
 *		Used:									
 *	Author		:	Jeff Sondermeyer
 *  Create Date	:	01/11/2002
 *	Modified	:	
 *----------------------------------------------*/
SetupEBIUAsynchMem:
	p0.l = EBIU_AMBCTL0 & 0xffff;
	p0.h = (EBIU_AMBCTL0 >> 16) & 0xffff;
	
	// Bank 0: Ardy =0, ArdyPol =0,  BTT =01(1), BST =01(1), BHT =10 (2), BRAT =0001 (1), BWAT = 0100(4).
	r0.l = 0xffc7;	//0x4194;	
	
	//Keep Default value for bank 1												
	r0.h = 0xffc2; 	
	[p0] = r0;
	SSYNC;

	p0.l = EBIU_AMBCTL1 & 0xffff;
	p0.h = (EBIU_AMBCTL1 >> 16) & 0xffff;

	// Bank 2: Ardy =1, ArdyPol =1,  BTT =01(1), BST =00(3), BHT =11 (3), BRAT =1111 (15), BWAT = 1111(15).
	r0.l = 0xffc7;	
	
	//Keep Default value for bank 3												
	r0.h = 0xffc2; 	
	[p0] = r0;
	SSYNC;
														
	
	p0.l = EBIU_AMGCTL & 0xffff;		
	p0.h = (EBIU_AMGCTL >> 16) & 0xffff;

	//Clockout: Enabled, Bank0,1,2,3: enabled; 32 bits data path enabled
	r0 = 0x0007 (z);	
	W[p0] = r0;			
	SSYNC;				

	RTS;

/*----------------------------------------------*
 *	Name		:	InitCommonDMA
 *	Function	:	Initialize some common stuff for DMA.
 					For ex: DP BASE address, End DMA DP etc.
 *	Register 	:
 *		In:			None		
 *		Out:		None
 *		Used:									
 *	Author		:	Jeff Sondermeyer
 *  Create Date	:	01/11/2002
 *	Modified	:	
 *----------------------------------------------*/
InitCommonDMA:

	//Initialize RXDMA count and TX DMA count
	p0.l		= RXDMACount;
	p0.h		= RXDMACount;
    r0			= 0;
	[p0++]		= r0;	//RX
	[p0]		= r0;	//TX

	//Init ending dp, all DMA DP point to this after transaction;
	p0.l	= DMAEndDp;
	p0.h	= DMAEndDp;
	r0.l	= 0;
	W[p0]	= r0.l;

	//Init Descriptor base pointer register */
	r0.l	= RXReadDMADp1;
	r0.h	= RXReadDMADp1;
	P0.L 	= DB_NDBP & 0xFFFF;
	P0.H 	= (DB_NDBP >> 16) & 0xFFFF;	
	W[p0]	= r0.h;

	RTS;

/*----------------------------------------------*
 *	Name		:	SetupRXDMA
 *	Function	:	Setup RX MemDMA for get data
 					From ADC
 *	Register 	:
 *		In:			None		
 *		Out:		None
 *		Used:									
 *	Author		:	Jeff Sondermeyer
 *  Create Date	:	01/11/2002
 *	Modified	:	
 *----------------------------------------------*/
SetupRXDMA:

	/* Source DMA */	
	//Read Descripter1 Setup
	p0.l		= RXReadDMADp1;
	p0.h		= RXReadDMADp1;

	//Configuration, 32bit, int disable
	r0.l		= 0x8009;		
	W[p0]		= r0.l;
	r0.l		= RX_DMA_LENGTH_SOURCE;
	W[p0+0x2]	= r0;
	r0.l		= IO_PORT_ADDR  & 0xffff;
	r0.h		= (IO_PORT_ADDR >> 16) & 0xffff;
	[p0+0x4]	= r0;
	r0.l		= RXReadDMADp2;
	W[p0+0x8]	= r0;

	//Setup Next DP pointer
	r0		= p0;
	P0.L 	= MDR_DND & 0xFFFF;
	P0.H 	= (MDR_DND >> 16) & 0xFFFF;	
	W[p0]	= r0.l;

	//Read Descripter2 Setup
	p0.l		= RXReadDMADp2;
	p0.h		= RXReadDMADp2;

	//Configuration, 32bit, int disable
	r0.l		= 0x8009;
	W[p0]		= r0.l;
	r0.l		= RX_DMA_LENGTH_SOURCE;
	W[p0+0x2]	= r0;
	r0.l		= IO_PORT_ADDR & 0xffff;
	r0.h		= (IO_PORT_ADDR >> 16) & 0xffff;
	[p0+0x4]	= r0;
	r0.l		= RXReadDMADp1;
	W[p0+0x8]	= r0;

	/* Destination DMA */
	// Write descriptor1 Setup 
	p0.l		= RXWriteDMADp1;
	p0.h		= RXWriteDMADp1;

	//Configuration, 32bit, int enabled on completion
	r0.l		= 0x800b;
	W[p0]		= r0.l;
	r0.l		= RX_DMA_LENGTH_DESTINATION;
	W[p0+0x2]	= r0;
	r0.l		= RXDMABufferA;
	r0.h		= RXDMABufferA;
	[p0+0x4]	= r0;
	r0.l		= RXWriteDMADp2;
	W[p0+0x8]	= r0;

	//Setup Next DP pointer
	r0		= p0;
	P0.L 	= MDW_DND & 0xFFFF;
	P0.H 	= (MDW_DND >> 16) & 0xFFFF;	
	W[p0]	= r0.l;

	// Write descriptor2 Setup 
	p0.l		= RXWriteDMADp2;
	p0.h		= RXWriteDMADp2;

	//Configuration, 32bit, int enabled on completion
	r0.l		= 0x800b;
	W[p0]		= r0.l;
	r0.l		= RX_DMA_LENGTH_DESTINATION;
	W[p0+0x2]	= r0;
	r0.l		= RXDMABufferB;
	r0.h		= RXDMABufferB;
	[p0+0x4]	= r0;
	r0.l		= RXWriteDMADp1;
	W[p0+0x8]	= r0;

	RTS;

/*----------------------------------------------*
 *	Name		:	StartRXDMA
 *	Function	:	Start RX MemDMA for get data
 					From ADC
 *	Register 	:
 *		In:			None		
 *		Out:		None
 *		Used:									
 *	Author		:	Jeff Sondermeyer
 *  Create Date	:	01/11/2002
 *	Modified	:	
 *----------------------------------------------*/

StartRXDMA:
	// Enable Destination
	P0.L 	= MDW_DCFG & 0xFFFF;
	P0.H 	= (MDW_DCFG >> 16) & 0xFFFF;	
	R0 		= W[P0];
	BITSET(R0,0);
	W[p0]	= R0.l;

	// Enable source
	P0.L 	= MDR_DCFG & 0xFFFF;
	P0.H 	= (MDR_DCFG >> 16) & 0xFFFF;	
	R0 		= W[P0];
	BITSET(R0,0);
	W[p0]	= r0.l;
	RTS;

/*----------------------------------------------*
 *	Name		:	SetupTXDMA
 *	Function	:	Setup TX MemDMA to send data
 					to DAC
 *	Register 	:
 *		In:			None		
 *		Out:		None
 *		Used:									
 *	Author		:	Jeff Sondermeyer
 *  Create Date	:	01/17/2002
 *	Modified	:	
 *----------------------------------------------*/
SetupTXDMA:

	/* Source DMA */	
	//Read Descripter1 Setup
	p0.l		= TXReadDMADp1;
	p0.h		= TXReadDMADp1;

	//Configuration, 32bit, int disable
	r0.l		= 0x8009;		
	W[p0]		= r0.l;
	r0.l		= TX_DMA_LENGTH_SOURCE;
	W[p0+0x2]	= r0;
	r0.l		= TXDMABufferA;
	r0.h		= TXDMABufferA;
	[p0+0x4]	= r0;
	r0.l		= TXReadDMADp2;
	W[p0+0x8]	= r0;

	//Setup Next DP pointer
	r0		= p0;
	P0.L 	= MDR_DND & 0xFFFF;
	P0.H 	= (MDR_DND >> 16) & 0xFFFF;	
	W[p0]	= r0.l;

	//Read Descripter2 Setup
	p0.l		= TXReadDMADp2;
	p0.h		= TXReadDMADp2;

	//Configuration, 32bit, int disable
	r0.l		= 0x8009;
	W[p0]		= r0.l;
	r0.l		= TX_DMA_LENGTH_SOURCE;
	W[p0+0x2]	= r0;
	r0.l		= TXDMABufferB;
	r0.h		= TXDMABufferB;
	[p0+0x4]	= r0;
	r0.l		= TXReadDMADp1;
	W[p0+0x8]	= r0;

	/* Destination DMA */
	// Write descriptor1 Setup 
	p0.l		= TXWriteDMADp1;
	p0.h		= TXWriteDMADp1;

	//Configuration, 32bit, int enabled on completion
	r0.l		= 0x800f;
	W[p0]		= r0.l;
	r0.l		= TX_DMA_LENGTH_DESTINATION;
	W[p0+0x2]	= r0;
	r0.l		= IO_PORT_ADDR  & 0xffff;
	r0.h		= (IO_PORT_ADDR >> 16) & 0xffff;
	[p0+0x4]	= r0;
	r0.l		= TXWriteDMADp2;
	W[p0+0x8]	= r0;

	//Setup Next DP pointer
	r0		= p0;
	P0.L 	= MDW_DND & 0xFFFF;
	P0.H 	= (MDW_DND >> 16) & 0xFFFF;	
	W[p0]	= r0.l;

	// Write descriptor2 Setup 
	p0.l		= TXWriteDMADp2;
	p0.h		= TXWriteDMADp2;

	//Configuration, 32bit, int enabled on completion
	r0.l		= 0x800f;
	W[p0]		= r0.l;
	r0.l		= TX_DMA_LENGTH_DESTINATION;
	W[p0+0x2]	= r0;
	r0.l		= IO_PORT_ADDR & 0xffff;
	r0.h		= (IO_PORT_ADDR >> 16) & 0xffff;
	[p0+0x4]	= r0;
	r0.l		= TXWriteDMADp1;
	W[p0+0x8]	= r0;

	RTS;

/*----------------------------------------------*
 *	Name		:	StartTXDMA
 *	Function	:	Start TX MemDMA to send data to DA
 *	Register 	:
 *		In:			None		
 *		Out:		None
 *		Used:									
 *	Author		:	Jeff Sondermeyer
 *  Create Date	:	01/11/2002
 *	Modified	:	
 *----------------------------------------------*/

StartTXDMA:
	// Enable Dest
	P0.L 	= MDW_DCFG & 0xFFFF;
	P0.H 	= (MDW_DCFG >> 16) & 0xFFFF;	
	R0 		= W[P0];
	BITSET(R0,0);
	W[p0]	= R0.l;

	// Enable source
	P0.L 	= MDR_DCFG & 0xFFFF;
	P0.H 	= (MDR_DCFG >> 16) & 0xFFFF;	
	R0 		= W[P0];
	BITSET(R0,0);
	W[p0]	= r0.l;
	RTS;


	
/*SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/
/*Start code Section*/
.section start_from_here;
	r0		= SetupOpMode(Z);
	r0.h	= SetupOpMode;
	p0		= r0;
	csync;
	jump	 (p0);
	nop;
	nop;

/*SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/
/*Stack  Section*/
.section ustack;   
.align 4;
.var UserStackSpace0[1000];
UserStack:
.var UserStackSpace1[4];

/*SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS*/
/*Data Section*/

// L2 bank 2 */
.section L2_B2_data;
/* MemDMA descriptor buffer */
//First RX 
.align 4;
.BYTE2	RXReadDMADp1[5];
.align 4;
.BYTE2	RXWriteDMADp1[5];

//First TX
.align 4;
.BYTE2	TXReadDMADp1[5];
.align 4;
.BYTE2	TXWriteDMADp1[5];


//END
.align 4;
.BYTE2	DMAEndDp[1];
.BYTE2	TimerToggle[1];
.align 4;
.BYTE4	RXDMACount[1];
.BYTE4	TXDMACount[1];
.align 4;
.BYTE2	SysMode[1];

// L2 Bank 3 */
.section L2_B3_data;
/* MemDMA descriptor buffer */
//Second RX
.align 4;
.BYTE2	RXReadDMADp2[5];
.align 4;
.BYTE2	RXWriteDMADp2[5];

//Second TX
.align 4;
.BYTE2	TXReadDMADp2[5];
.align 4;
.BYTE2	TXWriteDMADp2[5];


// L1 B bank, sub bank 0 */
.section L1_B0_data;
.align 4;
TXDMABufferA:
.BYTE4	RXDMABufferA[1024];

// L1 B bank, sub bank 1 */
.section L1_B1_data;
.align 4;
TXDMABufferB:
.BYTE4	RXDMABufferB[1024];

